#!/usr/bin/env python
import sys
import time
import logging
import os.path
import subprocess

import paleomix
import paleomix.yaml
import paleomix.logger
from paleomix.pipeline import Pypeline
from paleomix.common.console import print_err, print_info

import parts
from makefile import MakefileError, read_makefiles
from config import parse_config


_tools = {"filterMetaphlan":  ["nodes", "tools", "filterMetaphlan.py"],
          "get_stats":        ["nodes", "tools", "get_stats.py"],
          "metaphlan2krona":  ["nodes", "tools", "metaphlan2krona_2.py"],
          "reformat_taxa":    ["nodes", "tools", "reformat_taxa.py"],
          "doPcoa":           ["nodes", "tools", "statax_Rmodule", "doPcoa.R"],
          "doClust":          ["nodes", "tools", "statax_Rmodule", "doClust.R"],
          "doDiv":            ["nodes", "tools", "statax_Rmodule", "doDiv.R"],
          "doHeatmap":        ["nodes", "tools", "statax_Rmodule", "doHeatmap.R"]}


def run(config, args):
    logfile_template = time.strftime("metaBIT.%Y%m%d_%H%M%S_%%02i.log")
    paleomix.logger.initialize(config, logfile_template)
    logger = logging.getLogger(__name__)
    try:
        logger.info("Building metaBIT pipeline ...")
        makefiles = read_makefiles(args, config)
    except (MakefileError, paleomix.yaml.YAMLError, IOError), error:
        print_err("Error reading makefiles:",
                  "\n  %s:\n   " % (error.__class__.__name__,),
                  "\n    ".join(str(error).split("\n")),
                  file=sys.stderr)
        return 1

    # setups for the logfile
    pipeline = Pypeline(config=config)

    for makefile in makefiles:
        if not makefile.get('run_from_table'):
            rmdup_nodes, metaphlan_nodes, summary_node \
                = parts.profiling(makefile, config)

            pipeline.add_nodes(rmdup_nodes, summary_node)
            pipeline.add_nodes(parts.analyzing(makefile, config, metaphlan_nodes))
        else:
            pipeline.add_nodes(parts.analyzing(makefile, config))

    if config.list_output_files:
        logger.info("Printing output files ...")
        pipeline.print_output_files()
        return 0
    elif config.list_executables:
        logger.info("Printing required executables ...")
        pipeline.print_required_executables()
        return 0

    logger.info("MetaBIT running now!")
    if not pipeline.run(dry_run=config.dry_run,
                        max_threads=config.max_threads,
                        progress_ui=config.progress_ui):
        return 1

    return 0


def _print_usage():
    print_info("METAGENOMICS Pipeline %s\n"
               % (paleomix.__version__,))
    print_info("Usage:")
    print_info("  paleomix metabit makefile.yaml")


def main(argv):
    if argv and argv[0] in _tools:
        path = os.path.dirname(os.path.realpath(__file__))
        command = os.path.join(*[path] + _tools[argv[0]])
        return subprocess.call([command] + argv[1:])

    config, args = parse_config(argv)
    if not args:
        _print_usage()
        print_err("\nPlease specify at least one makefile!\n")
        return 1

    return run(config, args)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python


"""Parse arguments from command line and return a tuple (config, args)"""

#from glob import glob
#import os.path
import optparse

import paleomix
import paleomix.ui

from paleomix.config import PerHostConfig, PerHostValue, ConfigError


__foreword__ = """
metaBIT, an integrative and automated metagenomic pipeline for analysing
microbial profiles from high-throughput sequencing shotgun data

Guillaume Louvel, Clio Der Sarkissian, Ludovic Orlando.
Centre for GeoGenetics, Natural History Museum of Denmark, University of Copenhagen.
Version: %s
                              -----
""" % paleomix.__version__


__description__ = """%prog processes trimmed reads produced using shotgun
high through-put DNA sequencing, provided in fastq (or compressed gz/bz2), and
generates taxonomic microbial profiles as well as their statistical comparison
across samples, libraries, environments, etc."""


__epilog__ = """Please report bugs and suggestions for improvements to:
Guillaume Louvel (guillaume.louvel@ens.fr)
Clio Der Sarkissian (clio.dersarkissian@snm.ku.dk)
Kristian hanghoej (k.hanghoej@snm.ku.dk)
Ludovic Orlando (lorlando@snm.ku.dk)
"""


class metaBITHelpFormatter(optparse.IndentedHelpFormatter):
    def format_usage(self, usage):
        return usage + "\n"

    def format_epilog(self, epilog):
        return "\n" + epilog


def _run_config_parser(argv):
    # Helper class for optparse.OptionParser
    # allows reading from and writing to a config file
    per_host_cfg = PerHostConfig("metaBIT")

    usage_str    = ("Usage:\n  %prog [options] makefile.yaml  # yaml formatted file, "
                    "other extensions allowed\n"
                    "  %prog [options] --write-config # configure metaBIT "
                    "dependencies for future runs")
    version_str  = "%%prog %s" % (paleomix.__version__,)


    parser       = optparse.OptionParser(usage = __foreword__ + "\n" + usage_str,
                                         version = version_str,
                                         description = __description__,
                                         epilog = __epilog__,
                                         formatter=metaBITHelpFormatter(),
                                         prog="metaBIT")
    
    paleomix.ui.add_optiongroup(parser,
                                ui_default=PerHostValue("quiet"),
                                color_default=PerHostValue("on"))
    paleomix.logger.add_optiongroup(parser, default = PerHostValue("warning"))

    group  = optparse.OptionGroup(parser, "Scheduling")
    group.add_option("--bowtie2-max-threads", type = int, default = PerHostValue(2),
                     help = "Maximum number of threads to use per Bowtie 2 "
                     "instance [%default]")
    group.add_option("--metaphlan-max-threads", type = int, default = PerHostValue(2),
                     help = "Maximum number of threads to use per MetaPhlAn "
                     "instance [%default]")
    group.add_option("--max-threads", type = int, default = per_host_cfg.max_threads,
                     help = "Maximum number of threads to use in total [%default]")
    group.add_option("--dry-run", action = "store_true", default = False,
                     help = "If passed, only a dry-run in performed, the dependency "
                     "tree is printed, and no tasks are executed.")
    parser.add_option_group(group)

    group  = optparse.OptionGroup(parser, "Required paths")
    group.add_option("--jar-root", default = PerHostValue("~/install/jar_root",
                                                          is_path = True),
                     help = "Folder containing Picard JARs (http://picard.sf.net) " \
                     "[%default]")
    group.add_option("--temp-root", default = per_host_cfg.temp_root,
                     help = "Location for temporary files and folders [%default/]")
    group.add_option("--destination", default = None,
                     help = "The destination folder for result files. By default, "
                     "files will be placed in ./out_{makefile name}/")
    group.add_option("--metaphlan-path", default = PerHostValue("~/install/metaphlan",
                                                                is_path=True),
                     help = "Path to the folder containing metaphlan(2).py and " \
                     "the database [%default]")
    group.add_option("--lefse-path", default = PerHostValue("~/install/lefse",
                                                            is_path = True),
                     help = "Path to the LEfSe executables [%default] "
                     "(https://bitbucket.org/nsegata/lefse, "
                     "\"Metagenomic biomarker discovery and explanation\", "
                     "Segata et al. 2011)")
    parser.add_option_group(group)

    group  = optparse.OptionGroup(parser, "Misc")
    group.add_option("--jre-option", dest = "jre_options", action = "append",
                     default = PerHostValue([]),
                     help = "May be specified one or more times with options to be "
                            "passed to the JRE (Jave Runtime Environment); e.g. to "
                            "change the maximum amount of dedicated RAM (default is %default)")
    parser.add_option_group(group)

    group  = optparse.OptionGroup(parser, "Files and executables")
    group.add_option("--list-output-files", action = "store_true", default = False,
                     help = "List all files generated by metaBIT for a given makefile")
    group.add_option("--list-orphan-files", action = "store_true", default = False,
                     help = "List all files at destination not generated by the pipeline. " \
                            "This option is useful for rerunning metaBIT after editing the makefile from a previous analysis. If the same destination is required and the same name is given for the" 
                       " set of analyses, metaBIT will NOT erase the results of previous analyses,"
                       " even if new parameters are required. It will then report the list of such "
                       "files for traceability. Useful only for cleaning up after making changes "
                       "to a makefile.")
    group.add_option("--list-executables", action="store_true", default=False,
                     help="List all executables required by the pipeline, "
                          "with version requirements (if any).")
    parser.add_option_group(group)

    return per_host_cfg.parse_args(parser, argv)


def parse_config(argv):
    config, args = _run_config_parser(argv)
    paleomix.ui.set_ui_colors(config.ui_colors)

#    FIXME: Nuke?
#    if not glob( os.path.join(config.metaphlan_path, "metaphlan*.py")):
#        raise ConfigError(("Wrong metaphlan-path: %s\n'metaphlan*.py' not found"))

    if config.list_output_files and config.list_orphan_files:
        raise ConfigError("Error: Both --list-output-files and --list-orphan-files set!")

    return config, args



#!/usr/bin/env python

# =====================================================================
_description = """
 Conversion script: from MetaPhlAn output to Krona text input file
 Author: Daniel Brami (daniel.brami@gmail.com)
 modified for metaBIT"""
# =====================================================================


import sys
import argparse
import re


def convert(inputfile, outputfile, no_underscore=False):

    re_candidates = re.compile(r"s__|unclassified\t")
    re_replace = re.compile(r"\w__")
    re_bar = re.compile(r"\|")

    IN  = open(inputfile,'r')  if inputfile  else sys.stdin
    OUT = open(outputfile,'w') if outputfile else sys.stdout

    try:
        for line in IN:
            if len(line.split("\t")) > 2:
                print >> sys.stderr, \
                      "Error: Wrong input file (more than two fields)"
                return 1

            if re.search(re_candidates, line):
                x=re.sub(re_replace, '\t', line)
                x=re.sub(re_bar, '', x)
                x=re.sub(r'_', ' ', x) if no_underscore else x

                x_cells = x.split('\t')
                lineage = '\t'.join(x_cells[:-1])
                abundance = x_cells[-1].rstrip('\n') 

                OUT.write(abundance + '\t' + lineage + '\n')
   
    finally:
        if OUT != sys.stdout:
            OUT.close()
        if IN != sys.stdin:
            IN.close()

    return 0


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=_description,
                          formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('inputfile', type=str, nargs='?',
                        help='The input file is the MetaPhlAn standard result'
                             ' file. Default to stdin')
    parser.add_argument('outputfile', type=str, nargs='?',
                        help='the Krona output file name. Default to stdout' )
    parser.add_argument('--no-underscore', action='store_true',
                        help='convert underscores in fields to spaces')
    args = parser.parse_args()

    sys.exit(convert(args.inputfile, args.outputfile, args.no_underscore))




#!/usr/bin/env python

import os.path
import pysam

from paleomix.node import Node, CommandNode, NodeError
from paleomix.atomiccmd.command import AtomicCmd
from paleomix.atomiccmd.sets import \
     ParallelCmds
from paleomix.atomiccmd.builder import \
     AtomicCmdBuilder

import paleomix.common.versions as versions


_COMMON_CHECK = versions.GE(1, 1, 0)

SAMTOOLS_VERSION = versions.Requirement(
    call=("samtools", "--version"),
    search=r"samtools (\d+)\.(\d+)\.?(\d+)?",
    checks=_COMMON_CHECK)



class SortSamNode(CommandNode):
    """sorts SAM file and returns a BAM"""

    def __init__(self, input_file, output_file, dependencies=()):
        cmd_sam2bam = AtomicCmd(["samtools", "view", "-buS", "%(IN_SAM)s"],
                                IN_SAM = input_file,
                                OUT_STDOUT = AtomicCmd.PIPE,
                                CHECK_VERSION = SAMTOOLS_VERSION)
        cmd_sortbam = AtomicCmd(["samtools", "sort",
                                 "-T", "%(TEMP_OUT_PREFIX)s",
                                 "-o", "%(OUT_BAM)s",
                                 "-O", "bam",
                                 "-"],
                                IN_STDIN=cmd_sam2bam,
                                # when starting with TEMP_OUT, temp_root path
                                # is added automatically
                                TEMP_OUT_PREFIX="SamToBam",
                                OUT_BAM=output_file)

        description = "< samtools: view (sam to bam) and sort\n %s -> %s >" \
                      % (os.path.basename(input_file),
                         os.path.basename(output_file))

        CommandNode.__init__(self,
                             command      = ParallelCmds([cmd_sam2bam,
                                                          cmd_sortbam]),
                             description  = description,
                             #threads      = parameters.threads,
                             dependencies = dependencies)


class SortSamNode2(Node):
    """sort using pysam"""
    def __init__(self, input_file, output_file, dependencies=()):
        description = "< pysam sort (sam to bam)\n %s -> %s >" \
                      % (os.path.basename(input_file),
                         os.path.basename(output_file))
        Node.__init__(self,
                      description, 
                      input_files=input_file,
                      output_files=output_file,
                      dependencies=dependencies)
    
    def _run(self, config, temp):
        input_file, = self.input_files
        output_file, = self.output_files
        with pysam.AlignmentFile(input_file, 'r') as IN_SAM:
            reads = [read for read in IN_SAM]
            reads.sort(key=lambda read: read.rname)

            with pysam.AlignmentFile(output_file, 'wb', template=IN_BAM) as OUT_BAM:
                for read in reads:
                    OUT_BAM.write(read)


### Not used. This conversion is done in the _setup step of the MetaphlanNode.
class BAM2bowtie2outNode(Node):
    def __init__(self, input_file, output_file, dependencies=()):
        """'dependency' should be the rmdup_node"""

        description = "< Format SAM to bowtie2out format\n %s -> %s >" \
                      % (os.path.basename(input_file),
                         os.path.basename(output_file))

        Node.__init__(self,
                      description  = description,
                      #threads      = parameters.threads,
                      input_files  = input_file,
                      output_files = output_file,
                      #requirement  =
                      dependencies = dependencies)
    
    def _run(self, config, temp):
        input_file,  = self.input_files
        output_file, = self.output_files
        with open(output_file, 'w') as OUT_BOWTIE:
            with pysam.AlignmentFile(input_file, 'rb') as IN_BAM:
                references = IN_BAM.references
                for read in IN_BAM:
                    OUT_BOWTIE.write("%s\t%s\n" %(read.query_name,
                                                  references[read.tid]))


#class BAMIndexNode(CommandNode):
#    """Index a BAM file using 'samtools index'."""
#
#    def __init__(self, infile, dependencies=()):
#        cmd_index = AtomicCmd(["samtools", "index", "%(IN_BAM)s",
#                               "%(OUT_BAI)s"],
#                              IN_BAM=infile,
#                              OUT_BAI=swap_ext(infile, ".bai"),
#                              CHECK_SAM=SAMTOOLS_VERSION)
#
#        CommandNode.__init__(self,
#                             description="<BAMIndex: '%s'>" % (infile,),
#                             command=cmd_index,
#                             dependencies=dependencies)




#!/usr/bin/env python

import pysam
import os.path

from itertools import izip, izip_longest
import paleomix.common.fileutils as fileutils

from paleomix.common.utilities import safe_coerce_to_tuple
from paleomix.node import Node, CommandNode, NodeError
from paleomix.atomiccmd.builder import AtomicCmdBuilder, \
                                       create_customizable_cli_parameters, \
                                       use_customizable_cli_parameters

from tools.reformat_taxa import merge_tables

_tmp_inputfile = "tmp_bowtie2out.tsv"


class MetaphlanNode(CommandNode):
    @create_customizable_cli_parameters
    def customize(cls, metaphlan, input_files, output_file, threads=1,
                  dependencies=[], ID = ''):

        input_files = safe_coerce_to_tuple(input_files)
        input_files = list(input_files)
        
        call = [metaphlan,
                "--nproc", threads,
                "--input_type", "bowtie2out",
                "%(TEMP_OUT_FILE)s", "%(OUT_FILE)s"]
        
        cmd = AtomicCmdBuilder(call,
                               TEMP_OUT_FILE = _tmp_inputfile,
                               OUT_FILE = output_file)
        
        cmd.set_kwargs(**{"IN_FILE_%02i" %(i+1,): infile \
                                    for i,infile in enumerate(input_files)})
        
        return {"command": cmd}


    @use_customizable_cli_parameters
    def __init__(self, parameters):
        
        self.output_file = parameters.output_file
        
        if not parameters.ID:
            self.ID = os.path.dirname( \
                            parameters.output_file).replace("/", "_") or \
                      os.path.splitext(parameters.output_file)[0]
        else:
            self.ID = parameters.ID

        basenames = [os.path.basename(f) for f in parameters.input_files]
        
        desc = "<MetaPhlan: %r: %s -> %r>" %(self.ID,
                                            "\n".join(parameters.input_files), 
                                            parameters.output_file)
        CommandNode.__init__(self, parameters.command.finalize(),
                            description=desc,
                            threads=parameters.threads,
                            dependencies=parameters.dependencies)


    def _setup(self, _config, _temp):
        """Create a temporary file of the bowtie2out format from the input
        BAMs (merge if several):
        bowtie2out is the format returned by metaphlan when only running
        bowtie2. These are the 1st and 3d columns of the SAM returned by
        bowtie2 (QNAME and RNAME). Format required when using
        'metaphlan --input_type bowtie2out' """
        CommandNode._setup(self, _config, _temp)

        # Additional step: create the temporary "bowtie2out" file
        input_files = list(self.input_files)
        output_file = os.path.join(_temp, _tmp_inputfile)
        
        with open(output_file, 'w') as OUT_BOWTIE:
            for infile in input_files:
                with pysam.AlignmentFile(infile, 'rb') as IN_BAM:
                    references = IN_BAM.references
                    for read in IN_BAM:
                        OUT_BOWTIE.write("%s\t%s\n" %(read.query_name,
                                                      references[read.tid]))

        CommandNode._check_for_missing_files(self, output_file, "Temporary input file")


class MergeTablesNode(Node):
    def __init__(self, output_file, samplefiles, dependencies = (), **merge_opts):
        self.samplefiles = samplefiles
        self.output_file = output_file
        self.merge_opts = merge_opts

        input_files = [value[0] for value in samplefiles]

        desc = "<merge metaphlan tables: %s -> %r>" %(("\n"+" "*25).join(input_files),
                                                      os.path.basename(output_file))

        Node.__init__(self,
                      input_files = input_files,
                      output_files = output_file,
                      description = desc,
                      dependencies = dependencies)

    def _run(self, _config, _temp):
        output_file, = self.output_files
        output_file = os.path.join(_temp, os.path.basename(output_file))
        merge_tables(output_file, self.samplefiles, **self.merge_opts)

    def _teardown(self, _config, _temp):
        expected_files = [os.path.join(_temp, os.path.basename(f)) for f in \
                                                       list(self.output_files)]
        self._check_for_missing_files(expected_files, "output")
        for tmp_f, commited_f in zip(expected_files, list(self.output_files)):
            fileutils.move_file(tmp_f, commited_f)


class Metaphlan2LefseNode(Node):
    def __init__(self, input_file, output_file, sep='_', groups=None, dependencies=()):
        self.sep = sep
        if groups:
            self.groups = groups
        else:
            raise NodeError("No groups given to the LEfSe input, the LEfSe "
                            "cannot be run.")

        desc = "<Specify groups of samples in the merged metaphlan table>"
        Node.__init__(self, input_files = input_file,
                            output_files = output_file,
                            description = desc,
                            dependencies = dependencies)
    
    def _run(self, _config, _temp):

        input_file, = self.input_files
        output_file, = self.output_files
        output_file = os.path.join(_temp, os.path.basename(output_file))
        
        with open(input_file) as IN:
            ID_line = IN.readline()
            IDs = ID_line.rstrip().split("\t")
            # Warning: the user should preferably not use underscores in sample names
            #sample_names = [ID.split(self.sep)[0] for ID in IDs]

            order = [0] #the first column is for the row names
            allgroups = []
            for groups, samples in iterate_tree(self.groups):
                for s in samples:
                    try:
                        allgroups.append(groups)
                        order.append(IDs.index(s))
                    except ValueError as e: # when sample name not in the IDs
                        raise NodeError(e.message + \
                                "\nList of sample names: %s" % " ".join(IDs))

            with open(output_file, 'w') as OUT:
                grouplevel = "class"
                for groupline in izip_longest(*allgroups, fillvalue=''):
                    OUT.write(grouplevel + "\t" + "\t".join(groupline) + "\n")
                    grouplevel = "sub" + grouplevel

                OUT.write("\t".join(IDs[i] for i in order) + "\n")

                for line in IN:
                    line = line.rstrip().split("\t")
                    OUT.write("\t".join(line[i] for i in order) + "\n")


    def _teardown(self, _config, _temp):
        expected_files = [os.path.join(_temp, os.path.basename(f)) for f in \
                                                                list(self.output_files)]
        self._check_for_missing_files(expected_files, "output")
        for tmp_f, commited_f in zip(expected_files, list(self.output_files)):
            fileutils.move_file(tmp_f, commited_f)


def iterate_tree(tree, path=()):
    """tree is a nested dictionary. This function creates an iterator
    yielding the deepest elements of the tree with the corresponding path"""
    if isinstance(tree, dict):
        for key, value in tree.iteritems():
            for elem in iterate_tree(value, path + (key,)):
                yield elem
    else:
        yield path, tree



#!/usr/bin/env python

"""From a merged table of metaphlan abundances, creates the filtered data
divided by taxonomical levels"""


import re
import sys
import csv
import os.path

#taxlevels = ['k','p','c','o','f','g','s','t']
_intaxlevels='kpcofgst'

def dofilter(inputfile, outbase=None, filterout=1, keep_full_tax=False,
             taxlevels='kpcofgst', splitchar='|'):

    outbase = os.path.splitext(os.path.basename(inputfile))[0] if not outbase else outbase

    with open(inputfile) as IN:
        reader = csv.reader(IN, dialect='excel-tab')
        header = reader.next()
        n = len(header) - 1
        filtered_Data     = {tax_key: [header] for tax_key in _intaxlevels}
        filtered_samples  = {tax_key: [] for tax_key in _intaxlevels}
        filtered_taxa_nb  = {tax_key: [0] * n for tax_key in _intaxlevels}
        filtered_taxa_sum = {tax_key: [0] * n for tax_key in _intaxlevels}

        for tax_key, rowname, abundances in getassign_lines(reader, keep_full_tax,
                                                            splitchar=splitchar):

            row            = [0 if ab < filterout else ab for ab in abundances]
            low_abundances = [ab if ab < filterout else 0 for ab in abundances]
            islow_abundances = [low_ab > 0 for low_ab in low_abundances]

            filtered_taxa_nb[tax_key] = map(sum, zip(islow_abundances,
                                                     filtered_taxa_nb[tax_key]))
            filtered_taxa_sum[tax_key] = map(sum, zip(low_abundances,
                                                      filtered_taxa_sum[tax_key]))

            if not all(ab==0 for ab in row): 
                filtered_Data[tax_key].append([rowname] + row)
    
    for k, tax_key in enumerate(_intaxlevels):
        i = 1
        while i < len(filtered_Data[tax_key][0]) - 1:
            if all(row[i]==0 for row in filtered_Data[tax_key][1:]):
                sample = filtered_Data[tax_key][0][i]
                for child_tax in _intaxlevels[k:]:
                    filtered_samples[child_tax].append(sample)
                    filtered_Data[child_tax] = [row[:i]+row[i+1:] for row in \
                                                filtered_Data[child_tax]]
            else:
                i += 1

    for tax_key in taxlevels:
        with open(outbase + "_filtered_%s.tsv" % tax_key, 'w') as OUT:
            writer = csv.writer(OUT, dialect='excel-tab')
            writer.writerows(filtered_Data[tax_key])
    
    with open(outbase + "_filtered_samples.txt", 'w') as OUT:
        for tax_key in taxlevels:
            OUT.write(tax_key + ':\t' + '\t'.join(filtered_samples[tax_key]) + '\n')

    with open(outbase + "_filtered_taxa_nb.tsv", 'w') as OUT:
        writer = csv.writer(OUT, dialect='excel-tab')
        writer.writerow(header)
        for tax_key in taxlevels: 
            writer.writerow([tax_key] + filtered_taxa_nb[tax_key])

    with open(outbase + "_filtered_taxa_sum.tsv", 'w') as OUT:
        writer = csv.writer(OUT, dialect='excel-tab')
        writer.writerow(header)
        for tax_key in taxlevels:
            writer.writerow([tax_key] + filtered_taxa_sum[tax_key])
    

def getassign_lines(csvreader, keep_full_tax=False, intaxlevels=_intaxlevels,
                    splitchar='\|'):
    gettaxlevel = re.compile(r'([%s])__' % intaxlevels )
    for row in csvreader:
        fulltax = row.pop(0)
        gettax_key = gettaxlevel.findall(fulltax)
        if not gettax_key:
            tax_key = 'k'
            tax = fulltax
        else:
            tax_key = gettax_key[-1]
            gettax = re.compile(r'%s__([A-Za-z0-9_]+)(?:%s|$)' % (tax_key, splitchar))
            tax = fulltax if keep_full_tax else gettax.search(fulltax).group(1)
        yield tax_key, tax, [int(float(x)) if float(x)%1==0 else float(x) for x in row]


if __name__ == '__main__':

    import argparse

    parser = argparse.ArgumentParser()

    parser.add_argument("inputfile", help="merged metaphlan table")
    parser.add_argument("-o", "--outbase",
                        help=("base name of output files. default: "
                              "basename of input file"))
    parser.add_argument("-f", "--filterout", type=float, default=1,
                        help=("threshold for filtering out taxa: taxa under the "
                              "threshold in all samples will be trashed "
                              "[%(default)s %%]"))
    parser.add_argument("--keep-full-tax", action='store_true', default=False,
                        help=("Keep the original taxonomical assignment as row "
                            "name, not only the name at a given taxonomical level"))
    parser.add_argument("--splitchar", "-s", default="|",
                        help=("Character separating taxonomical levels in the "
                              "input row names [%(default)r]"))
    parser.add_argument('-t', '--taxlevels', default='kpcofgst',
                        help="taxonomical levels kept in the output [%(default)s]")
        
    args = parser.parse_args()
    dofilter(**vars(args))



#!/usr/bin/env python

import re
import os
import os.path
from glob import glob
from itertools import izip_longest

from paleomix.common.makefile import MakefileError
from paleomix.common.console  import print_info


_trimmed_types = ['Collapsed', 'Pair1', 'Pair2', 'Singles']

# This is AdapterRemoval default output (see man AdapterRemoval).
# If using Paleomix bam_pipeline, the files are gzipped/bzipped
_reg_PE = {'Collapsed': re.compile(r'^.*\.collapsed(\.truncated|)(\.gz|\.bz2|)$'),
           'Pair1':     re.compile(r'.*\.pair1\.truncated\.(gz|bz2)$'),
           'Pair2':     re.compile(r'.*\.pair2\.truncated\.(gz|bz2)$'),
           'Singles':   re.compile(r'.*\.singleton\.truncated\.(gz|bz2)$')}
_reg_SE = re.compile(r'.*\.truncated\.(gz|bz2)$')


class InputLibraryError(MakefileError):
    """Raised if library does not meet requirements
    (wrong path, wrong files, etc)"""


def _fmt_info(path, files):
    files_1, files_2 = zip(*files) or ([], [])
    if any(files_2):
        files_1 = [f.replace("pair1", "pair[12]") for f in files_1]
    files = [f.replace(path+"/", "") for f in files_1]
    return ("\n" + " "*16).join(files)


def check_lib(path):
    """Automatically find trimmed reads files in 'path'"""
    if not os.path.isdir(path):
        raise InputLibraryError("The specified library path is not a "
                                "regular existing directory:\n%s" %path)
    path = os.path.abspath(path)
    files = os.listdir(path)
    lib = {k: [] for k in ('Collapsed', 'Paired', 'Singles')}
    
    isPE = False

    for root, dirs, files in os.walk(path):
        tmp = {}
        for k in _trimmed_types:
            tmp[k] = [os.path.join(root, f) for f in files if _reg_PE[k].match(f)]

        if not any(tmp.values()):
            lib['Singles'] += [(os.path.join(root, f), None) for f in files if _reg_SE.match(f)]

        else:
            lib['Collapsed'] += [(x, None) for x in tmp['Collapsed']]
            lib['Singles']   += [(x, None) for x in tmp['Singles']]
            lib['Paired']    += zip(tmp['Pair1'], tmp['Pair2'])
            isPE |= True

    print_info("\nAuto-detect files in library folder:  %s" % path)

    if not isPE and len(lib['Singles']) > 0 :
        lib.pop('Collapsed')
        lib.pop('Paired')
    elif not isPE:
        raise InputLibraryError("  Found no pre-trimmed fastq files (expected "
                                "file names:\n  default output of "
                                "AdapterRemoval/Paleomix)\n")

    print_info("  %s detected." % (isPE*"PE" or "SE",))
    for k,v in lib.iteritems():
            print_info("  - {:<11} {}".format(k + " :", _fmt_info(path, v)))
    
    return lib



#!/usr/bin/env python


"""
./get_stats output_file lib_dir1 [,libdir2 [,libdirN]]

Gather stats (nb of reads) from input files and mapping and
remove-duplicates in a table.
"""


import re
import sys
import pysam
import os.path
import shlex
from subprocess import check_output
from glob import glob


class bowtie2statsError(Exception):
    def __init__(self, value):
        self.parameter = value
    def __str__(self):
        return repr(self.parameter)


def get_bowtie2stats(file_):
    if not file_:
        return (None, None)

    with open(file_) as IN:
        lines = IN.readlines()
    
    reg_blank = re.compile(r'^$')
    regex = [
        # tuple (name of the retrieved number, pattern)
        ('tot',  r'^(\d+) reads(?:; of these:)?$'),
        ('p',    r'^  \d+ \([0-9\.]+%\) were (paired|unpaired); of these:$'),
        ('pc0',  r'^    (\d+) \([0-9\.]+%\) aligned concordantly 0 times$'),
        ('pc1',  r'^    (\d+) \([0-9\.]+%\) aligned concordantly exactly 1 time$'),
        ('pc2',  r'^    (\d+) \([0-9\.]+%\) aligned concordantly >1 times$'),
        ('_',    r'^    ----$'),
        ('pc0_2',r'^    (\d+) pairs aligned concordantly 0 times; of these:$'),
        ('pd1',  r'^      (\d+) \([0-9\.]+%\) aligned discordantly 1 time$'),
        ('pd2',  r'^      (\d+) \([0-9\.]+%\) aligned discordantly >1 times$'), #probably never happens
        ('_',   r'^    ----$'),
        ('s',   r'^    (\d+) pairs aligned 0 times concordantly or discordantly; of these:$'),
        ('sp',   r'^      (\d+) mates make up the pairs; of these:$'),
        ('s0',   r' {4,8}(\d+) \([0-9\.]+%\) aligned 0 times$'),
        ('s1',   r' {4,8}(\d+) \([0-9\.]+%\) aligned exactly 1 time$'),
        ('s2',   r' {4,8}(\d+) \([0-9\.]+%\) aligned >1 times$'),
        ('rate', r'^([0-9\.]+)% overall alignment rate$')]

    regex = [(name, re.compile(pattern)) for name,pattern in regex]
    matched = {}
    unmatched = []

    # read the file until it finds the first match of the expected message
    name0, reg0 = regex[0]
    
    if not lines:
        raise bowtie2statsError("File %s is empty." % file_)

    match0 = reg0.match(lines[0])
    while not match0 and len(lines)>1:
        if not reg_blank.match(lines[0]):
            unmatched += [lines.pop(0)]
        else:
            lines.pop(0)
        match0 = reg0.match(lines[0])
    if not match0:
        raise bowtie2statsError("File %s does not contain bowtie2 read counts." % file_)

    # then match lines in their expected order
    for name, reg in regex:
        match = reg.match(lines[0])
        if match:
            lines.pop(0)
            matched[name] = match.group(1) if name != '_' else match.group()
        else:
            matched[name] = 0  # to allow summing these names
    
    unmatched += lines  # lines should be empty if everything has matched

    if unmatched:
        unmatched = unmatched[:10] + [" + %s more..." %(len(unmatched)-10,)] if len(unmatched)>10 else unmatched
        sys.stderr.write("In %s those lines did not match:\n%s\n" %(file_, "\n".join(unmatched) ))

    tot = int(matched['tot'])
    if matched['p'] == 'paired':
        tot *= 2

    mapped  = int(matched['s1']) + int(matched['s2'])
    mapped += 2 * (int(matched['pc1']) + int(matched['pc2']))

    return tot, mapped



def get_readcount(bamfile, cmd="samtools view -c {}".format):
    if bamfile:
        readcount = check_output(shlex.split(cmd(bamfile)))
        return(int(readcount.rstrip("\n")), )
    else:
        return (None, )
    

def _get_readcount(bamfile):
    if bamfile:
        readcount = pysam.view("-c", bamfile)[0]
        return (int(readcount),)
    else:
        return (None,)


def grab_files(pattern, directories):
    for libdir in directories:
        found = glob(os.path.join(libdir, pattern))
        yield found[0] if found else None  # only keeps one file per directory


def make_lines(ttable, fields, func, fieldtuple, elemlist):
    lines = zip(*[func(e) for e in elemlist])
    for line, field in zip(lines, fieldtuple):
        if any(line):
            ttable[field] = line
            fields.append(field)


def gather_stats(output_file, directories, IDs=None):
    """IDs is a list of sample_lib names corresponding to the directories"""
    
    if not IDs:
        IDs = directories

    ttable = {"sample_lib": IDs}
    fields = ["sample_lib"]

    for trimmed_type in ['collapsed', 'paired', 'singles']:
        make_lines(ttable,
                   fields,
                   func = get_bowtie2stats,
                   fieldtuple = (trimmed_type+'.tot', trimmed_type+'.mapped'),
                   elemlist = list(grab_files('*.%s.bowtie2out.stats' % trimmed_type,
                                              directories)))
    
        make_lines(ttable,
                   fields,
                   func = get_readcount,
                   fieldtuple = (trimmed_type + '.no_dup',),
                   elemlist = list(grab_files(
                                   '*.%s.bowtie2out*.sorted.no_dup.bam' % trimmed_type,
                                              directories)))

    # compute totals if more than 1 trimmed_type
    for result_type in ['tot', 'mapped', 'no_dup']:
        fromfields = [t+result_type for t in ['collapsed.', 'paired.', 'singles.']]
        fromfields = [f for f in fromfields if ttable.get(f)]
        if len(fromfields) > 1:
            make_lines(ttable,
                       fields,
                       func = lambda tup :(sum(x for x in tup if not x is None),),
                       fieldtuple = ('sum.'+result_type,),
                       elemlist = zip(*(ttable[f] for f in fromfields)))
   
    with open(output_file, 'w') as OUT:
        OUT.write('\t'.join(fields) + '\n')
        for line in zip(*(ttable[field] for field in fields)):
            line = [str(count) for count in line]
            OUT.write("\t".join(line) + '\n')
    

if __name__ == '__main__':
    
    if len(sys.argv) > 2:
        output_file = sys.argv[1]
        directories = []
        for arg in sys.argv[2:]:
            directories.extend(glob(arg))

        gather_stats(output_file, directories)
    else:
        sys.exit(__doc__)




#!/usr/bin/env python


import sys
import os.path
import argparse


description = ("Merge multiple Metaphlan output tables, by filling with 0 "
               "in tables where a given taxon hasn't been found.\n"
               "Can remerge merged tables")
_sep='\t'


def _all_indices(V, elem):
    """ returns all indices of elem in list V"""
    start = 0
    indices = []
    while elem in V[start:]:
        start += V[start:].index(elem) + 1
        indices.append(start-1)
    return indices


def _line2tuple(line, iCol=0):
    """iCol is the column number of the ID names.
    returns: (ID, abundances)
    types:   (str, list)"""
    line = line.rstrip().split('\t')
    return line[iCol], line[:iCol] + line[iCol+1:]


def merge_tables(outputfile, samplefiles, idstr='ID', keep_original_IDs=True):
    """Each samplefile must be a list of size 1 or 2:
    samplefile = [filename, name] or just [filename]"""

    # add a default name when name is not specified (i.e len(f)==1)
    samplefiles = [f + [os.path.splitext(os.path.basename(f[0]))[0]]*(2-len(f)) \
                   for f in samplefiles]
    #remove identical files while keeping the order:
    samplefiles = [f for i,f in enumerate(samplefiles) if f not in samplefiles[:i]]
    files, names = zip(*samplefiles)
    
    # if same names for different file names (default names on files with same
    # basename), restore default name to full file name.
    for name in set(names):
        positions = _all_indices(names, name)
        if len(positions) > 1:
            for duplicated in positions:
                samplefiles[duplicated][1] = samplefiles[duplicated][0]

    # Build merged table of all samplefiles 
    samples = []
    taxa = set()
    for file_, samplename in samplefiles:
        with open(file_) as IN:
            table = dict(_line2tuple(line) for line in IN)
        
        if keep_original_IDs and table.get(idstr):
            table[idstr] = table.get(idstr)
        else:
            table[idstr] = [samplename]

        keys = set(table.keys())
        newtaxa    = keys - taxa
        oldtaxa    = taxa - keys
        
        for previous_table in samples:
            dim = len(previous_table[idstr])
            previous_table.update({ new: ["0"]*dim  for new in newtaxa})
        
        table.update({ old: ["0"]*len(table[idstr])  for old in oldtaxa})

        samples.append(table)
        taxa |= keys

    taxa.remove(idstr)
    
    OUT = sys.stdout if outputfile == "-" else open(outputfile, 'w')
    try:
        for taxon in [idstr] + sorted(taxa):
            OUT.write(taxon + \
                      _sep  + \
                      _sep.join(
                          _sep.join(sample[taxon]) for sample in samples) + \
                      "\n")
    finally:
        if outputfile == "-":
            OUT.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument("-o", "--outfile", default="-", help="default: stdout")
    parser.add_argument("input_table", nargs="+")
    parser.add_argument("--idstring", default="ID",
                        help=("row name of the ID line (sample name)."
                              "NOTE: it is '#SampleID' in metaphlan2 tables"))
    parser.add_argument("--trash-original-IDs", action='store_false',
                        dest="keep_original_IDs",
                        help="trash the ID names if present in input tables")

    args = parser.parse_args()
    
    merge_tables(args.outfile, [arg.split(",") for arg in args.input_table],
                 idstr=args.idstring, 
                 keep_original_IDs=args.keep_original_IDs)




#!/usr/bin/env python

""" For each library, builds the nodes corresponding to the sequence of:
    - Bowtie2,
    - picardtools SortSam,
    - bam_rmdup / MarkDuplicates,
    - Samtools view (to remove marked duplicates)
    - Metaphlan.
"""

import os.path
import nodes.tools as tools

from paleomix.common.console import \
    print_warn

from paleomix.common.fileutils import \
    swap_ext

from paleomix.atomiccmd.builder import \
    apply_options

from paleomix.nodes.bowtie2 import \
    Bowtie2Node
from nodes.remove_pair2 import \
    removePair2Node
from paleomix.nodes.picard import \
    MarkDuplicatesNode
from paleomix.nodes.commands import \
    FilterCollapsedBAMNode
from nodes.metaphlan import \
    MetaphlanNode, \
    MergeTablesNode, \
    Metaphlan2LefseNode

# for krona, statax, lefse:
from nodes.execute import \
    ExecNode, \
    SimpleExecNode, \
    ExecModuleNode


_trimmed_types = ("Collapsed", "Paired", "Singles")


def profiling(makefile, config, sep='_'):
    """returns nodes from the profiling, ready to be used by the Rscripts part
    sep is the string separating sample name and library name in the output"""
    rmdup_nodes_list = []
    metaphlan_nodes_list = []
    summary_params = {'dependencies': [],
                      'input_files': [],
                      'IDs': [],
                      'directories': [],
                      'function_in_args': None,
                      'function_out_args': ['output_file'],
                      'output_file': os.path.join(makefile['OutDir'],
                                                  'summary_readcounts.tsv')}
    summary_params['output_files'] = summary_params['output_file']

    for (Samplename, Sample) in makefile['Samples'].iteritems():
        metaphlan_params = []

        for (Libname, inputlib) in Sample.iteritems():
            folder = os.path.join(makefile['OutDir'], Samplename, Libname)
            ID = Samplename + sep + Libname
            summary_params['directories'].append(folder)
            summary_params['IDs'].append(ID)

            if Samplename not in makefile['Metaphlan']['Pool']:
                output_file = os.path.join(folder, 'taxa.tsv')
                metaphlan_params.append({"dependencies": [],
                                         "input_files": [],
                                         "output_file": output_file,
                                         "ID": ID})

            elif not metaphlan_params:
                print_warn("Pooling sample: " + Samplename)

                folder = os.path.join(makefile['OutDir'], Samplename)

                output_file = os.path.join(folder, 'taxa.tsv')
                metaphlan_params.append({"dependencies": [],
                                         "input_files": [],
                                         "output_file": output_file,
                                         "ID": Samplename})

            for trimmed_type, input_files in inputlib.iteritems():
                dependencies = ()
                output_name = 'reads.' + trimmed_type.lower()
                input_files_1, input_files_2 = zip(*input_files)
                bowtie2_options = makefile['Bowtie2']

                if not all(input_files_2):
                    if any(input_files_2):
                        # There cannot be a different number of pair1 and pair2
                        raise BaseException
                    else:
                        input_files_2 = None
                else:
                    bowtie2_options.update({PEopt: True for PEopt in
                                           ['--no-discordant']})
                                            #'--no-overlap']})

                output_base = os.path.join(makefile['OutDir'],
                                           Samplename, Libname,
                                           output_name + '.bowtie2out')
                output_file = output_base + '.bam'
                log_file = output_base + '.stats'

                # Bowtie2
                alignment_node = _build_bowtie2_nodes(config=config,
                                                      input_files_1=input_files_1,
                                                      input_files_2=input_files_2,
                                                      output_file=output_file,
                                                      log_file=log_file,
                                                      dependencies=dependencies,
                                                      options=makefile['Bowtie2'])
                summary_params['input_files'].append(log_file)
                summary_params['dependencies'].append(alignment_node)

                if trimmed_type == 'Paired':
                    input_file = output_file
                    output_file = swap_ext(input_file, 'mate1.bam')
                    output_file_trash = swap_ext(input_file, 'mate2.bam')
                    remove_pair2_node = removePair2Node(input_file,
                                                        output_file,
                                                        output_file_trash,
                                                        dependencies=alignment_node)
                    dependencies = (remove_pair2_node,)
                else:
                    dependencies = (alignment_node,)

                # Remove duplicates
                input_file = output_file
                output_file = input_file.replace('.bam', '.filtered.bam')

                rmdup_cls = FilterCollapsedBAMNode if trimmed_type == 'Collapsed' \
                            else MarkDuplicatesNode
                rmdup_node = rmdup_cls(config       = config,
                                       input_bams    = (input_file,),
                                       output_bam   = output_file,
                                       keep_dupes   = False,
                                       dependencies = dependencies)
                summary_params['input_files'].append(output_file)
                summary_params['dependencies'].append(rmdup_node)

                if Samplename+":"+Libname in makefile['Metaphlan']['Exclude']:
                    print_warn("Excluding  %s:%s (%s)" % (Samplename, Libname,
                                                          trimmed_type))
                    rmdup_nodes_list.append(rmdup_node)
                else:
                    metaphlan_params[-1]['input_files'].append(output_file)
                    metaphlan_params[-1]['dependencies'].append(rmdup_node)
                # END of for-loop on each trimmed_type
            # END of the for-loop on each library
        #still in the for-loop on each sample:
        for params in metaphlan_params:
            metaphlan_nodes_list.append(_build_metaphlan_node(config, params,
                                                       makefile['Metaphlan']))

    summary_node = ExecModuleNode('nodes.tools.get_stats',
                                  'gather_stats',
                                  description = 'Summarize read counts',
                                  **summary_params)

    return rmdup_nodes_list, metaphlan_nodes_list, summary_node


def analyzing(makefile, config, metaphlan_nodes=(), group='all'):
    nodes = []
    statax_dep = ()
    runonly = makefile.get('run_from_table')

    if metaphlan_nodes:
        statax_in = os.path.join(makefile['OutDir'], group+'_taxa.tsv')
        merge_in = [[node.output_file, node.ID] for node in metaphlan_nodes]
        merge_samples_node = MergeTablesNode(output_file = statax_in,
                                             samplefiles = merge_in,
                                             dependencies = metaphlan_nodes,
                                             idstr = 'ID',
                                             keep_original_IDs = False)
        nodes.append(merge_samples_node)
        statax_dep += (merge_samples_node,)

    elif not runonly:
        # Metaphlan nodes are expected if statax is not set in runonly mode
        raise RuntimeError
    else:
        statax_in = runonly

    rename_taxlevels = makefile['Statax'].pop('rename_taxlevels')

    statax_path = os.path.join(os.path.abspath(tools.__path__[0]),
                               "statax_Rmodule")

    # list of list of files for merging (one list per merge)
    merge_list = []
    # one per different merging
    filter_nodes = []

    for run_name, run in makefile['Statax'].iteritems():
        run_outdir = os.path.join(makefile['OutDir'], 'statax', run_name)

        merge = run.get("merge")
        if merge not in merge_list:
            merge_list.append(merge)
            if merge:
                filter_infile = os.path.join(run_outdir, run_name + '.tsv')
                samplefiles = [[f] for f in [statax_in] + merge]
                merge_node = MergeTablesNode(
                                output_file = filter_infile,
                                samplefiles = samplefiles,
                                dependencies = statax_dep)
                                #keep_original_IDs = False)
                filter_dep = merge_node
            else:
                filter_infile = statax_in
                filter_dep = statax_dep

            filter_outbase = os.path.join(run_outdir, 'tables', run_name)
            filter_taxlevels = 'kpcogfst' if makefile['Metaphlan'].get('--mpa_pkl') \
                                          else 'kpcofgs'
            filter_outfiles = [filter_outbase + '_filtered_' + L + '.tsv' for L in \
                                                                       filter_taxlevels]
            filter_outfiles += [filter_outbase + '_filtered_samples.txt',
                                filter_outbase + '_filtered_taxa_sum.tsv',
                                filter_outbase + '_filtered_taxa_nb.tsv']
            filter_node = ExecModuleNode("nodes.tools.filterMetaphlan",
                                         "dofilter",
                                         input_files=filter_infile,
                                         output_files=filter_outfiles,
                                         dependencies = filter_dep,
                                         function_in_args=["inputfile"],
                                         function_out_args=None,
                                         function_out_tmp="outbase",
                                         outbase = filter_outbase,
                                         filterout = run['filterout'],
                                         taxlevels=filter_taxlevels)

            filter_nodes.append(filter_node)
            nodes.append(filter_node)
            run_dep = filter_node

            doDiv = run.get('doDiv')
            if doDiv is not None:
                div_node = SimpleExecNode(os.path.join(statax_path, 'doDiv.R'),
                                          description = '%s: doDiv' % run_name,
                                          input_file = filter_infile,
                                          output_file=os.path.join(run_outdir,
                                              run_name + "_diversities.tsv"),
                                          options=doDiv,
                                          dependencies = filter_dep)
                nodes.append(div_node)

        else: # merge in merge_list
            i = merge_list.index(merge)
            run_dep = filter_nodes[i]

        for L in run['taxlevels']:
            longL = rename_taxlevels[L]
            for operation in ['doBarplot', 'doHeatmap', 'doPcoa', 'doClust']:
                op = run.get(operation)
                if op is not None:
                    call = [os.path.join(statax_path, operation+".R"),
                            "%(IN_FILE)s"]
                    builder_kwargs = {'IN_FILE':
                            run_dep.function_kwargs['outbase'] + '_filtered_' + L + '.tsv'}
                    op_node = ExecNode.customize(call,
                                description="%s: %s (%s)" %(run_name, operation, longL),
                                threads=op.get('--ncores', 1),
                                dependencies=run_dep,
                                builder_kwargs=builder_kwargs)
                    out_base = os.path.join(run_outdir, run_name + '_' + \
                                operation.replace('do','').lower() + \
                                '_' + longL)
                    if operation in ['doBarplot', 'doHeatmap']:
                        op_node.command.add_value("%(OUT_PDF)s")
                        op_node.command.set_kwargs(OUT_PDF = out_base+'.pdf')
                        op_node.command.set_option('--title',
                            "%s abundances (level: %s)" %(run_name, longL))
                        op_node.command.set_option('--taxon_title', longL)
                    else:
                        op_node.command.set_option('--pdf', "%(OUT_PDF)s")
                        op_node.command.set_option('--tsv', "%(OUT_TSV)s")
                        op_node.command.set_kwargs(OUT_PDF= out_base + '.pdf',
                                                   OUT_TSV= out_base + '.tsv')
                        if operation == 'doClust':
                            op_node.command.set_option('--RData', "%(OUT_RDATA)s")
                            op_node.command.set_kwargs(OUT_RDATA=out_base+'.RData')

                    apply_options(op_node.command, op)
                    nodes.append(op_node.build_node())

    # Krona
    if makefile['Krona']['run'] and not runonly:
        output_base = os.path.join(makefile['OutDir'], 'krona')
        krona_input_files = []
        krona_dep = []
        for node in metaphlan_nodes:
            krona_infile = os.path.join(output_base, node.ID + ".krona.in")
            convert_node = ExecModuleNode("nodes.tools.metaphlan2krona_2",
                                          "convert",
                                          input_files = node.output_file,
                                          output_files = krona_infile,
                                          dependencies = node,
                                          description="convert metaphlan out -> krona in",
                                          function_in_args = ["inputfile"],
                                          function_out_args = ["outputfile"],
                                          no_underscore=True)

            krona_input_files.append(krona_infile)
            krona_dep.append(convert_node)

        output_file = os.path.join(output_base, group + '_taxa.krona.html')
        krona_node = ExecNode.customize(("ktImportText", '-o', "%(OUT_FILE)s"),
                               description="<krona: ktImportText -> %s>" % output_file,
                               dependencies = krona_dep,
                               builder_kwargs={"OUT_FILE": output_file})
        krona_node.command.set_option("-a", True, fixed=False)
        apply_options(krona_node.command, makefile['Krona'])
        krona_node.command.add_multiple_values(krona_input_files)

        nodes.append(krona_node.build_node())

    # LEfSe
    if makefile['Lefse']['run']:

        if makefile['Lefse'].get('merge'):
            input_file = os.path.join(makefile['OutDir'], makefile['Lefse']['outdir'],
                        os.path.basename(statax_in).replace('.tsv', '_merged.tsv'))
            samplefiles = [[f] for f in [statax_in] + makefile['Lefse']['merge']]
            lefse_merge_node = MergeTablesNode(output_file = input_file,
                                         samplefiles = samplefiles,
                                         dependencies = statax_dep)
                                         #keep_original_IDs = False)
            statax_dep = lefse_merge_node
        else:
            input_file = statax_in

        # format input by including groups on top of the sample names
        pre_lefse_input = os.path.join(makefile['OutDir'], makefile['Lefse']['outdir'],
                        os.path.basename(input_file).replace('.tsv', '.lefse.in.tsv'))

        metaphlan2lefse_node = Metaphlan2LefseNode(input_file = input_file,
                                                   output_file = pre_lefse_input,
                                                   groups = makefile['Lefse']['Groups'],
                                                   dependencies = statax_dep)

        run_lefse_input = pre_lefse_input.replace('lefse.in.tsv', 'lefse.in')

        subclass_nb = max((len(path) for path,_ in \
                                          _iterate_tree(makefile['Lefse']['Groups'])))
        format_options={"-c": 1,
                        "-s": 2 if subclass_nb==2 else -1,
                        "-u": 3 if subclass_nb==2 else 2}
        format_options.update(makefile["Lefse"]["format_input"])

        format_input_node = SimpleExecNode(os.path.join(config.lefse_path,
                                                        "format_input.py"),
                                           pre_lefse_input,
                                           run_lefse_input,
                                           options = format_options,
                                           dependencies = metaphlan2lefse_node)

        run_lefse_output = run_lefse_input.replace('lefse.in', 'lefse.out')
        run_lefse_node = SimpleExecNode(os.path.join(config.lefse_path, "run_lefse.py"),
                                  run_lefse_input,
                                  run_lefse_output,
                                  makefile["Lefse"]["run_lefse"],
                                  dependencies = format_input_node)

        fmt = makefile['Lefse']['format']
        for step in ['plot_res', 'plot_cladogram', 'plot_features']:
            if not makefile['Lefse'][step].has_key('--format'):
                makefile['Lefse'][step]['--format'] = fmt

        plot_res_node = SimpleExecNode(os.path.join(config.lefse_path, "plot_res.py"),
                                  run_lefse_output,
                                  run_lefse_output.replace('lefse.out', 'lefse.plot_res.'+fmt),
                                  makefile["Lefse"]["plot_res"],
                                  dependencies = run_lefse_node)
        plot_clad_node = SimpleExecNode(os.path.join(config.lefse_path, "plot_cladogram.py"),
                                  run_lefse_output,
                                  run_lefse_output.replace('lefse.out','lefse.cladogram.'+fmt),
                                  makefile["Lefse"]["plot_cladogram"],
                                  dependencies = run_lefse_node)

        plot_features_obj = ExecNode.customize(os.path.join(config.lefse_path,
                                                            "plot_features.py"),
                                               #description="Lefse: plot_features",
                                               dependencies = (metaphlan2lefse_node,
                                                           run_lefse_node))
        plot_features_obj.command.add_value(run_lefse_input) # dataset file
        plot_features_obj.command.add_value(run_lefse_output)
        plot_features_obj.command.add_value(run_lefse_output.replace('out', 'biomarkers.zip')) # output file
        plot_features_obj.command.set_option("--archive", "zip")
        apply_options(plot_features_obj.command, makefile['Lefse']['plot_features'])

        plot_features_node = plot_features_obj.build_node()

        nodes.extend([run_lefse_node,plot_clad_node,plot_res_node,plot_features_node])

    return nodes


def _build_bowtie2_nodes(config, input_files_1, input_files_2, output_file,
                         log_file, dependencies, options):

    prefix = os.path.join(config.metaphlan_path, "db_v20", "mpa_v20_m200")
    params = Bowtie2Node.customize(input_file_1=input_files_1,
                                   input_file_2=input_files_2,
                                   output_file=output_file,
                                   reference=None,
                                   prefix=prefix,
                                   log_file=log_file,
                                   threads=config.bowtie2_max_threads,
                                   dependencies=dependencies)

    apply_options(params.commands['aln'], {"--no-unal": True})
    apply_options(params.commands['aln'], options)

    return params.build_node()


def _build_metaphlan_node(config, params, options):
    if not params['output_file']:
        raise RuntimeError("No input given to metaphlan, you made a mistake somewhere.\n"
                           "have you excluded all the libraries of sample %s ?" %Samplename)
    else:
        metaphlan_script = os.path.join(config.metaphlan_path, "metaphlan2.py")
        node = MetaphlanNode.customize(metaphlan_script,
                                       threads=config.metaphlan_max_threads,
                                       **params)
        apply_options(node.command, options)
    return node.build_node()


def _iterate_tree(tree, path=()):
    """Iterate over a tree of nested dictionary, flattening the output:
    each most inside value is returned with the set of keys leading to it (path)"""
    if isinstance(tree, dict):
        for key, value in tree.iteritems():
            for elem in _iterate_tree(value, path+(key,)):
                yield elem
    else:
        yield path, tree




#!/usr/bin/env python

""" 
    Filter mapped paired-end data:
    if the two pairs mapped to the same reference, remove one of them,
    in order to avoid counting twice when estimating taxon abundance.
"""

import os.path
import pysam

from paleomix.node import Node


class removePair2Node(Node):
    def __init__(self, input_file, output_file, output_file_trash, dependencies=()):
        """'dependency' should be the bowtie2Node"""
        
        self.outsam = output_file
        self.outtrash = output_file_trash

        description = "< Remove pair2 %s -> %s >" \
                      % (os.path.basename(input_file),
                         os.path.basename(output_file))

        Node.__init__(self,
                      description  = description,
                      input_files  = input_file,
                      output_files = [output_file, output_file_trash],
                      dependencies = dependencies)
    
    def _run(self, config, temp):
        input_file, = self.input_files

        processed_qnames = []
        
        with pysam.AlignmentFile(input_file, 'r') as IN_SAM:
            with pysam.AlignmentFile(self.outsam, 'wh', template=IN_SAM) as OUT_SAM: 
                with pysam.AlignmentFile(self.outtrash, 'wh', template=IN_SAM) as OUT_TRASH: 
                    for read in IN_SAM:
                        if read.query_name not in processed_qnames :
                            processed_qnames.append(read.query_name)
                            OUT_SAM.write(read)
                        else:
                            OUT_TRASH.write(read)




#!/usr/bin/env python
import os.path
import importlib


from paleomix.node import Node, CommandNode
from paleomix.atomiccmd.builder import \
    AtomicCmdBuilder, \
    use_customizable_cli_parameters, \
    create_customizable_cli_parameters
from paleomix.common.fileutils import \
    describe_files, \
    move_file, \
    reroot_path
from paleomix.common.utilities import \
    safe_coerce_to_tuple


class SimpleExecNode(CommandNode):
    """Basic CommandNode to execute command-line scripts"""
    def __init__(self, executable, input_file, output_file, options={},
                 description=None, dependencies=()):

        cmd = AtomicCmdBuilder([executable, "%(IN_FILE)s", "%(OUT_FILE)s"],
                               IN_FILE=input_file,
                               OUT_FILE=output_file)

        for opt, value in options.iteritems():
            cmd.set_option(opt, value)

        if description is None:
            description = "<%r : %r -> %r>" % (os.path.basename(executable),
                                               input_file, output_file)

        CommandNode.__init__(self,
                             command=cmd.finalize(),
                             description=description,
                             dependencies=dependencies)


class ExecNode(CommandNode):
    @create_customizable_cli_parameters
    def customize(cls, call, description=None, threads=1,
                  dependencies=(), builder_kwargs={}):

        if not description:
            call = safe_coerce_to_tuple(call)
            description = "<%r>" % " ".join(call)

        cmd = AtomicCmdBuilder(call, **builder_kwargs)

        return {"command": cmd,
                "description": description}

    @use_customizable_cli_parameters
    def __init__(self, params):
        CommandNode.__init__(self,
                             params.command.finalize(),
                             params.description,
                             params.threads,
                             params.dependencies)


class ExecModuleNode(Node):
    """run a python function as a Node"""
    def __init__(self, module, function, input_files, output_files,
                 dependencies=(), description=None,
                 function_in_args="input_files",
                 function_out_args="output_files",
                 function_out_tmp=None,
                 **function_kwargs):
        self.module = 'paleomix.tools.metabit.' + module
        self.function = function
        self.function_in_args = function_in_args
        self.function_out_args = function_out_args
        self.function_kwargs = function_kwargs
        # args that must redirect to tmp dir
        # typically output directories
        self.function_out_tmp = function_out_tmp

        if not description:
            description = module + '.' + function

        description = "<%s: %s -> %s>" % (description,
                                          describe_files(input_files),
                                          describe_files(output_files))

        Node.__init__(self,
                      description=description,
                      input_files=input_files,
                      output_files=output_files,
                      dependencies=dependencies)

    def _run(self, config, temp):
        module = importlib.import_module(self.module)
        func = getattr(module, self.function)

        input_files = list(self.input_files)
        output_files = [reroot_path(temp, filename)
                        for filename in self.output_files]

        # resolve conflict in case 'input_files' is also an argument for the function
        #if "input_files" in func.func_code.co_varnames:
        #    self.function_kwargs.update({"input_files": input_files})
        if self.function_in_args:
            if isinstance(self.function_in_args, list):
                for in_arg, in_file in zip(self.function_in_args, input_files):
                    self.function_kwargs.update({in_arg: in_file})
            else:
                self.function_kwargs.update({self.function_in_args: input_files})

        #if "output_files" in func.func_code.co_varnames:
        #    self.function_kwargs.update({"output_files": output_files})
        if self.function_out_args:
            if isinstance(self.function_out_args, list):
                for out_arg, out_file in zip(self.function_out_args, output_files):
                    self.function_kwargs.update({out_arg: out_file})
            else:
                self.function_kwargs.update({self.function_out_args: output_files})

        if self.function_out_tmp:
            if isinstance(self.function_out_tmp, str):
                self.function_out_tmp = [self.function_out_tmp]

            for out_tmp in self.function_out_tmp:
                self.function_kwargs[out_tmp] \
                    = reroot_path(temp, self.function_kwargs[out_tmp])

        func(**self.function_kwargs)

    def _teardown(self, _config, temp):
        expected_files = [reroot_path(temp, filename)
                          for filename in self.output_files]

        self._check_for_missing_files(expected_files, "output")
        for tmp_f, commited_f in zip(expected_files, list(self.output_files)):
            move_file(tmp_f, commited_f)
